/*
 *  Copyright (C) 2008, Pedro Carlos da Silva Lara
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.   
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#ifndef _HEADER_BASECRYPT_H_
#define _HEADER_BASECRYPT_H_

#include <vector>

#include "defines.h"
#include "dhkey.h"
#include "point.h"
#include "ecutil.h"

using std::vector;

using ec::Point;
using ec::crypt::EcUtil;
using ec::crypt::DH_PublicKey;
using ec::crypt::DH_PrivateKey;


namespace ec {
namespace crypt {

/**
 * Interface para servir de base para algoritmos
 * de criptografia assimetrica
 *
 * Ex: ElGamal, RSA
 */

class BaseCrypt {
public:
       
    template< typename T > vector< byte > encrypt(const PublicKey< T >& pub, const vector< byte >& msg);
      
    template< typename T > 
    vector< byte > dencrypt(const PublicKey< T >& pub, const vector< byte >& msg);
       
};

/**
 * Interface para servir de base para algoritmos
 * de troca de chaves (Key Exchange)
 *
 * Ex: Diffie-Helman
 */

class BaseKeyExchange {

public:
    
    /**
     * Calcula o valor para enviar ao destinatario, este
     * metodo e usado em algoritmos do tipo Diffie-Hellman
     * onde a chave e gerada sem que qualquer das partes 
     * a conheca anteriormente
     * 
     */
    
    virtual vector< byte > encrypt();
    
    /**
     * Encripta a mensagem "keyToShare" usando a chave publica 
     * do destinatario "pub".
     * 
     * @param pub
     *     chave publica do destinatario
     * 
     * @param keyToShare 
     *     chave que sera compartilhada
     *
     * @return
     *     chave (mensagem) encriptada
     */
     
    template< typename T > 
    vector< byte > encrypt(const PublicKey< T >& pub, vector< byte > keyToShare);
    
    /**
     * Obtem a chave privada compartilhada atraves do metodo encrypt da
     * origem da chave
     * 
     * @param keyEncrypted
     *     chave obtida no metodo toSend do usuario que 
     *     quer estabelecer uma chave privada compartilhada
     * @return 
     *     a chave privada compartilhada    
     *
     */
    
    virtual vector< byte > getSharedKey(const vector< byte >& keyEncrypted);
    
    /**
     * Retorna a chave publica
     *
     */
     
    template< typename T > 
    PublicKey< T > getPublicKey();
    
    
};

} // namespace ec
} // namespace crypt

#endif // _HEADER_BASECRYPT_H_
