/*
 *  Copyright (C) 2008, Pedro Carlos da Silva Lara
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.   
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _HEADER_DIFFIEHELLMAN_H_
#define _HEADER_DIFFIEHELLMAN_H_

#include <vector>

#include "defines.h"
#include "dhkey.h"
#include "point.h"
#include "ecutil.h"
#include "basecrypt.h"

using std::vector;

using ec::Point;
using ec::crypt::EcUtil;
using ec::crypt::DH_PublicKey;
using ec::crypt::DH_PrivateKey;


namespace ec {
namespace crypt {
	
/**
 * Esta classe implementa o protocolo Diffie-Hellman sobre 
 * curvas elipticas para troca de chaves 
 */ 

class DiffieHellman : public BaseKeyExchange {
public:
    
    /**
     * @param privateKey
     *     chave privada
     */ 
           
    DiffieHellman( const DH_PublicKey& publicKey,
                   const DH_PrivateKey& privateKey)
        : _publicKey( publicKey ), _privateKey( privateKey ) {}
       
    
    virtual ~DiffieHellman() {}
    
    
    /**
     * Calcula o valor para enviar ao destinatario
     * 
     * @return
     *     A chave privada vezes a chave publica que e um objeto do tipo 
     *     Point
     */
        
    vector< byte > encrypt() { 
        Point P = _publicKey.getKey();
        P = P * _privateKey.getKey();
        return EcUtil::toVectorBytes( P, true );                    
    }
    
    /**
     * Obtem a chave privada compartinhada
     * @param bytes
     *     valor recebido do usuario que deseja estabelecer uma
     *     chave privada compartilhada
     * @return 
     *     chave privada compartilhada
     */
    
    vector< byte > getSharedKey( const vector< byte >& bytes ) { 
        Point P = EcUtil::toPoint( bytes );
        P = P * _privateKey.getKey();
        return EcUtil::toVectorBytes( P, true );             
    }
    
    /**
     * Retorna a chave publica
     *
     */
     
    PublicKey< Point > getPublicKey() {
        return _publicKey;           
    }
    
private:

    DH_PublicKey   _publicKey;         
    DH_PrivateKey  _privateKey;
    
};

} // namespace ec
} // namespace crypt

#endif /* _HEADER_DIFFIEHELLMAN_H_ */
