/*
 *  Copyright (C) 2008, Pedro Carlos da Silva Lara
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.   
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <gmpxx.h>
#include <gmp.h>
#include "key.h"
#include "point.h"
#include "curve.h"

using ec::Point;
using ec::Curve;

Point::Point(const mpz_class& x, 
             const mpz_class& y, 
             const Curve& curve, 
             bool infty) : 
                   _x( x ), _y( y ), _curve( curve ) {
	_infty = infty;
}

Point::Point(const mpz_class& x, 
             const mpz_class& y, 
             const Curve& curve) : 
                   _x( x ), _y( y ), _curve( curve ) {
	_infty = false;
}

Point Point::operator+(const Point& P) {
    if( *this == P )
        return _double();
    if( _infty )
        return P;
    if( P.getInfty() )
        return *this;

    mpz_class tmp = P.getX() - _x;
    mpz_class invert = 0;      

    if( mpz_invert( invert.get_mpz_t(), 
                    tmp.get_mpz_t(), 
                    _curve.getP().get_mpz_t() ) == 0 )
        return InftyPoint;
    
    mpz_class t = ( P.getY() - _y )*invert;
        
    mpz_class xRet = t*t - _x - P.getX();
    mpz_class yRet = t*( _x - xRet) - _y;
    
    mpz_mod( xRet.get_mpz_t(), xRet.get_mpz_t(), _curve.getP().get_mpz_t() );
    mpz_mod( yRet.get_mpz_t(), yRet.get_mpz_t(), _curve.getP().get_mpz_t() );
    
    return Point( xRet, yRet, _curve );
}

Point Point::_double() {
    if( _infty )
        return InftyPoint;
    
    mpz_class tmp = 2*_y;
    mpz_class invert = 0;
    
    if( mpz_invert( invert.get_mpz_t(),
                    tmp.get_mpz_t(),
                    _curve.getP().get_mpz_t() ) == 0 )
        // Ponto no infinito            
        return Point( _curve );
    
    mpz_class t = ( 3*_x*_x + _curve.getA() ) * invert;
    mpz_class xRet = t*t - 2*_x;
    mpz_class yRet = t*( _x - xRet) - _y;
     
    mpz_mod( xRet.get_mpz_t(), xRet.get_mpz_t(), _curve.getP().get_mpz_t() );
    mpz_mod( yRet.get_mpz_t(), yRet.get_mpz_t(), _curve.getP().get_mpz_t() );
    
    return Point( xRet, yRet, _curve );
}

Point Point::operator*(const mpz_class& K) {
    mpz_class k ( K );
    Point P = *this;
    Point Q = InftyPoint;
    while( k != 0 ) {
        if( mpz_tstbit( k.get_mpz_t(), 0 ) == 1 )
            Q = Q + P;
        P = P._double();
        k /= 2;     
    }
    return Q;              
}
