/*
 *  Copyright (C) 2008, Pedro Carlos da Silva Lara
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.   
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#ifndef _POINT_H_
#define _POINT_H_
#include <string>
#include <gmpxx.h>
#include "curve.h"

using ec::Curve;

#define InftyPoint Point( Curve() );

namespace ec {
	
/**
 * Esta classe abstrai as funcionalidades de um 
 * ponto pertencente a uma curva eliptica.
 *
 */

class Point {

public:

    /**
     * Principal construtor
     * 
     * @param x 
     *     primeira coordenada
     * @param y 
     *     segunda coordenada
     * @param infty
     * 	   true se e um ponto no infinito, false caso contrario
     * @param curve 
     *     curva que este ponto pertence
     */
    
    Point(const mpz_class& x, 
          const mpz_class& y, 
          const Curve& curve,
          bool infty);
    
	/**
     * Principal construtor, constroi um ponto
     * que nao seja um ponto no infinito
     * 
     * @param x 
     *     primeira coordenada
     * @param y 
     *     segunda coordenada
     * @param curve 
     *     curva que este ponto pertence
     */
    
    Point(const mpz_class& x, 
          const mpz_class& y, 
          const Curve& curve);
    
    /**
     * Constroi um ponto no infinito
     *
     * @param curve 
     *     curva que este ponto pertence
     */
    
    Point( const Curve& curve ) : _curve( curve ) {
    	_infty = true;
    }
    
    /**
     * Construtor por copia
     * @param P 
     *     objeto que sera copiado
     */
    
    Point(const Point& P) { *this = P; }
    
    /**
     * Construtor default
     * 
     */ 
    
    Point() {}
    virtual ~Point() {}
    
    /**
     * @return 
     *     retorna a cordenada X
     */    
    
    mpz_class getX() const        { return _x; }
    
    /**
     * @return 
     *     retorna a cordenada Y
     */
    
    mpz_class getY() const        { return _y; }
    
    /**
     * Verifica se e um ponto no infinito
     * 
     * @return
     *     true caso seja um ponto no infinito, false caso contrario
     */
    bool getInfty()  const        { return _infty; }
    
    /**
     * Retorna a curva em questao
     *
     * @return
     *     A curva que o ponto pertence
     */
     
    Curve getCurve() const        { return _curve; }
    
    /**
     * Principal operacao: 'Multiplicacao por escalar'. Multiplica
     * o ponto *this por K.
     *
     * @param K
     *     inteiro que se deseja multiplicar por *this
     * @return
     *     o resultado da multiplicacao do ponto *this por K         
     */
    
    Point operator* (const mpz_class& K);
    
     /**
     * Verifica se dois objetos do tipo Point sao iguais
     *
     */
         
    bool operator== (const Point& P) {
         return ( _x == P.getX() &&
                  _y == P.getY() &&
                  _infty == P.getInfty() &&
                  _curve == P.getCurve() );
    }
    
    /**
     * Verifica se dois objetos do tipo Point sao diferentes
     *
     */
     
    bool operator!= (const Point& P) { return !(*this == P); }
    
    /**
     * @return
     *     retorna uma string formatada com o ponto *this
     *
     */
    
    std::string getString() const { 
        if(_infty) 
            return "infty";
        else {
            char ret[1000];
            gmp_sprintf(ret, "(%Zd, %Zd)", _x.get_mpz_t(), _y.get_mpz_t()); 
            return ret;
        }
    }
    
protected:
    
    /**
     * 
     * Operacao de soma entre pontos
     *
     * @param P
     *     ponto que sera somado ao ponto *this
     *
     * @return
     *     a soma de *this + P
     * 
     */
           
    Point operator+ (const Point& P);
    
    
private:

    Point _double();
    
    Point _double(const Curve& curve);
    
    mpz_class _x;
    mpz_class _y;
    bool _infty;
    Curve _curve;

};
}// namespace ec
#endif /* _POINT_H_ */
